// ----------------------------------------------------------------------------
// Decompose.java
// The "decompose" command.
// $Id: Decompose.java,v 1.2 2006/03/29 15:49:47 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.*;
import org.w3c.dom.*;
import fmrisc.Proving.*;

/**
 * 
 * command "decompose": decompose proof state by repeated skolemization
 * and flattening.
 *
 */
public final class Decompose extends ProofCommandBase
{
  public final static String Name = "decompose";
  private static ProofCommand flatten = new Flatten();
  private static ProofCommand skolemize = new Skolemize();
  
  /***************************************************************************
   * create a "decompose" command
   **************************************************************************/
  public Decompose()
  {
    super(Name);
  }
  
  /****************************************************************************
   * process command "decompose": 
   * @param state the proof state to which the command is applied
   * @return null or array of single new proof state.
   ***************************************************************************/
  protected ProofState[] processCore(ProofState state)
  {
    ProofState original = state;
    ProofState old = null;    
    // after stabilization, decompose and try again
    boolean simplified;
    do
    {
      simplified = false;
      if (old == state)
      {
        state.simplify(false);
        simplified = true;
      }
      old = state;
      ProofState[] state0 = skolemize.process(state);
      if (state0 != null) 
      {
        state = state0[0];
        //state.decompose();
      }
      ProofState[] state1 = flatten.process(state);
      if (state1 != null) 
      {
        state = state1[0];
        //state.decompose();
      }
    }
    while (!(old == state && simplified));
    if (state == original)
      return null;
    state.setParent(original);
    return new ProofState[] { state };
  }
  
  /***************************************************************************
   * Convert DOM node to command.
   * @param node the DOM representation of the command.
   * @return the command (null, if an error occured).
   **************************************************************************/
  public static Command newCommand(Node node)
  {
    return new Decompose();
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
  }
}
