// ----------------------------------------------------------------------------
// FormulaDeclaration.java
// Handling of formula declarations.
// $Id: FormulaDeclaration.java,v 1.7 2006/02/23 19:51:04 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;
import fmrisc.Semantics.*;

/**
 * Handling of type declarations.
 */
public class FormulaDeclaration extends DeclarationBase
{
  private FormulaDeclIdentifier name; // the declared name
  private Expression formula;         // the formula associated to the name

  /*****************************************************************************
   * construct formula declaration with denoted name and formula
   * @param name the declared name
   * @param formula the formula associated to the name 
   ****************************************************************************/
  public FormulaDeclaration(FormulaDeclIdentifier name, Expression formula)
  {
    this.name = name;
    this.formula = formula;
  }

  /*****************************************************************************
   * returns declaration name
   * @return the name
   ****************************************************************************/
  public FormulaDeclIdentifier getFormulaDeclIdentifier()
  {
    return name;
  }

  /*****************************************************************************
   * returns declaration name
   * @return the name
   ****************************************************************************/
  public Identifier getIdentifier()
  {
    return name;
  }

  /*****************************************************************************
   * returns declaration formula
   * @return the formula
   ****************************************************************************/
  public Expression getFormula()
  {
    return formula;
  }

  /*****************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * @param out the stream on which the text is written
   ****************************************************************************/
  public void printCore(PrintWriter out)
  {
    name.print(out);
    out.print(": FORMULA ");
    formula.print(out);
  }
  
  /****************************************************************************
   * returns symbol
   * @return the symbol (null, if not yet determined)
   ***************************************************************************/
  public Symbol getSymbol()
  {
    return name.getSymbol();
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null)
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    AST name0 = name.accept(visitor);
    AST formula0 = formula.accept(visitor);
    if (!visitor.isCloned()) return null;
    return Construct.formulaDeclaration(
      (FormulaDeclIdentifier)name0, (Expression)formula0);
  }
}
