// ----------------------------------------------------------------------------
// Number.java
// Handling of numbers.
// $Id: Number.java,v 1.13 2006/02/23 19:51:07 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;

/**
 * Handling of numbers.
 */
public final class Number extends ASTBase implements Expression
{
  //the digit representation of the number
  private String digits;

  /****************************************************************************
   * construct number from digit representation
   * @param digits the digit representation
   ***************************************************************************/
  public Number(String digits)
  {
    this.digits = digits;
  }
  
  /***************************************************************************
   * get digit representation of number
   * @return the digit representation
   **************************************************************************/
  public String getDigits()
  {
    return digits;
  }
  
  /***************************************************************************
   * get binding level for printing (lower numbers mean greater binding power)
   * @return the priority level
   **************************************************************************/
  public int getPriority()
  {
    return 0;
  }
  
  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(digits);
  }
  
  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * Overrides ASTBase.print such that parentheses are never printed
   * around the number.
   * @param out the stream on which the text is written
   **************************************************************************/
  public void print(PrintWriter out)
  {
    printCore(out);
  }
  
  /****************************************************************************
   * print expression, use parentheses if priority is greater than treshold
   * @param treshold priority level of enclosing expression
   * @param out writer to print expression on
   ***************************************************************************/
  public void printPriority(int treshold, PrintWriter out)
  {
    print(out);
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Expression instantiate()
  {
    return this;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be not null)
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    if (!visitor.isCloned()) return null;
    return this;
  }
}
