// ----------------------------------------------------------------------------
// TypeDeclIdentifier.java
// Handling of identifiers used in type declarations
// $Id: TypeDeclIdentifier.java,v 1.6 2006/02/23 19:51:04 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import fmrisc.Semantics.*;

import java.io.*;

/**
 * Handling of identifiers used in type declarations
 */
public final class TypeDeclIdentifier extends Identifier
{
  private TypeSymbol symbol; // symbol in value table linked to identifier
  private TypeDeclIdentifier substitution; // substitution for instantiation
  
  /****************************************************************************
   * construct identifier from name
   * @param name the text representation of the identifier
   ***************************************************************************/
  public TypeDeclIdentifier(String name)
  {
    super(name);
    symbol = new TypeSymbol(this);
  }
  
  /****************************************************************************
   * returns type symbol
   * @return the type symbol
   ***************************************************************************/
  public TypeSymbol getSymbol()
  {
    return symbol;
  }
 
  /****************************************************************************
   * returns substitution identifier for instantiation
   * @return the substitution identifier
   ***************************************************************************/
  public TypeDeclIdentifier getSubstitution()
  {
    return substitution;
  }
  
  /****************************************************************************
   * set substitution type for instantiation
   * @param substitution the substitution type
   ***************************************************************************/
  public void setSubstitution(TypeDeclIdentifier substitution)
  {
    this.substitution = substitution;
  }
  
  /***************************************************************************
   * reset substitution field
   **************************************************************************/
  public void resetSubstitution()
  {
    substitution = null;
  }
  
  /***************************************************************************
   * returns instantiated copy of identifier
   * @return the copy of the identifier
   **************************************************************************/
  public TypeDeclIdentifier instantiate()
  {
    TypeDeclIdentifier ident = Construct.typeDeclIdentifier(getName());
    TypeSymbol symbol0 = ident.getSymbol();
    symbol0.instantiate(symbol);
    return ident;
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    super.printCore(out);
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * Overrides ASTBase.print such that parentheses are never printed
   * around the number.
   * @param out the stream on which the text is written
   **************************************************************************/
  public void print(PrintWriter out)
  {
    super.print(out);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null).
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    if (!visitor.isCloned()) return null;
    return this.instantiate();
  }
}